@@ -1,15 +1,17 @@
-import json
+import sys
 import time
 import logging
 import subprocess
 import pandas as pd
 import datetime
-from os.path import join, split, isfile
+from os.path import join, split
 from d3m_interface.basic_ta3 import BasicTA3
+from d3m_interface.data_converter import is_d3m_format, convert_d3m_format
 
-logging.basicConfig(level=logging.ERROR, format='%(asctime)s %(levelname)s %(message)s')
+
+logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s', stream=sys.stdout)
 logger = logging.getLogger(__name__)
-pd.set_option('display.max_colwidth', -1)
+pd.set_option('display.max_colwidth', None)
 
 TA2_DOCKER_IMAGES = {'NYU': 'registry.gitlab.com/vida-nyu/d3m/ta2:latest', 'TAMU': 'dmartinez05/tamuta2:latest',
                      'CMU': 'registry.datadrivendiscovery.org/sheath/cmu-ta2:latest'}
@@ -23,95 +25,106 @@ IGNORE_SUMMARY_PRIMITIVES = {'d3m.primitives.data_transformation.construct_predi
 
 class Automl:
 
-    def __init__(self, output_folder, ta2='NYU'):
+    def __init__(self, output_folder, ta2_id='NYU'):
+        if ta2_id not in TA2_DOCKER_IMAGES:
+            logger.error('TA2 "%s" does not exist' % ta2_id)
+            return
+
         self.output_folder = output_folder
-        self.ta2 = ta2
-        self.pipelines = []
+        self.ta2_id = ta2_id
+        self.pipelines = {}
+        self.ta2 = None
         self.ta3 = None
-        self.container = None
         self.dataset = None
         self.leaderboard = None
+        self.problem_config = None
 
-    def search_pipelines(self, dataset_path, time_bound=10):
-        self.start_ta2(dataset_path, self.output_folder)
-        self.dataset = dataset_path
-        dataset_train_path = '/input/dataset/TRAIN/dataset_TRAIN/datasetDoc.json'
-        problem_path = join(dataset_path, 'problem_TRAIN/problemDoc.json')
+    def search_pipelines(self, dataset, time_bound, target=None, metric='f1Macro', task_keywords=['classification']):
+        suffix = 'TRAIN'
+        if not is_d3m_format(dataset, suffix):
+            self.problem_config = {'target_name': target, 'metric': metric, 'task_keywords': task_keywords}
+            dataset = convert_d3m_format(dataset, self.output_folder, self.problem_config, suffix)
+
+        self.dataset = split(dataset)[0]
+        self.start_ta2()
+
+        dataset_in_container = '/input/dataset/TRAIN/dataset_TRAIN/datasetDoc.json'
+        problem_path = join(dataset, 'problem_TRAIN/problemDoc.json')
         start_time = datetime.datetime.utcnow()
-        pipelines = self.ta3.do_search(dataset_train_path, problem_path, time_bound)
+        pipelines = self.ta3.do_search(dataset_in_container, problem_path, time_bound)
 
         for pipeline in pipelines:
             end_time = datetime.datetime.utcnow()
-            # TODO: do_describe should return the whole pipeline, but there is an issue in decode_pipeline_description
-            pipeline_json_id = self.ta3.do_describe(pipeline['id'])
-
-            with open(join(self.output_folder, pipeline['search_id'], 'pipelines_searched', '%s.json' % pipeline_json_id)) as fin:
-                pipeline_json = json.load(fin)
-                summary_pipeline = self.get_summary_pipeline(pipeline_json)
-                pipeline['json_representation'] = pipeline_json
-                pipeline['summary'] = summary_pipeline
-                pipeline['found_time'] = end_time.isoformat() + 'Z'
-
+            pipeline_json = self.ta3.do_describe(pipeline['id'], pipeline['search_id'], self.output_folder)
+            summary_pipeline = self.get_summary_pipeline(pipeline_json)
+            pipeline['json_representation'] = pipeline_json
+            pipeline['summary'] = summary_pipeline
+            pipeline['found_time'] = end_time.isoformat() + 'Z'
             duration = str(end_time - start_time)
-            print('Found pipeline, id=%s, %s=%s, time=%s' %
-                  (pipeline['id'], pipeline['metric'], pipeline['score'], duration))
-            self.pipelines.append(pipeline)
+            logger.info('Found pipeline, id=%s, %s=%s, time=%s' %
+                        (pipeline['id'], pipeline['metric'], pipeline['score'], duration))
+            self.pipelines[pipeline['id']] = pipeline
 
         if len(self.pipelines) > 0:
             leaderboard = []
-            sorted_pipelines = sorted(self.pipelines, key=lambda x: x['normalized_score'], reverse=True)
+            sorted_pipelines = sorted(self.pipelines.values(), key=lambda x: x['normalized_score'], reverse=True)
             metric = sorted_pipelines[0]['metric']
             for position, pipeline_data in enumerate(sorted_pipelines, 1):
                 leaderboard.append([position, pipeline_data['id'], pipeline_data['summary'],  pipeline_data['score']])
 
             self.leaderboard = pd.DataFrame(leaderboard, columns=['ranking', 'id', 'summary', metric])
 
-        return self.pipelines
+        return self.pipelines.values()
 
     def train(self, solution_id):
-        dataset_train_path = '/input/dataset/TRAIN/dataset_TRAIN/datasetDoc.json'
-        solution_ids = {p['id'] for p in self.pipelines}
+        dataset_in_container = '/input/dataset/TRAIN/dataset_TRAIN/datasetDoc.json'
 
-        if solution_id not in solution_ids:
-            print('Pipeline id=%s does not exist' % solution_id)
+        if solution_id not in self.pipelines:
+            logger.error('Pipeline id=%s does not exist' % solution_id)
             return
 
-        print('Training model...')
-        fitted_solution_id = self.ta3.do_train(solution_id, dataset_train_path)
-        fitted_solution = None  # TODO: Call to LoadFittedSolution
+        logger.info('Training model...')
+        fitted_solution_id = self.ta3.do_train(solution_id, dataset_in_container)
+        fitted_solution = None  # TODO: Call to LoadFittedSolution, but TA2 could not have implemented it yet
         model = {fitted_solution_id: fitted_solution}
-        print('Training finished!')
+        logger.info('Training finished!')
 
         return model
 
-    def test(self, model, test_dataset_path):
-        dataset_test_path = '/input/dataset/TEST/dataset_TEST/datasetDoc.json'
+    def test(self, model, test_dataset):
+        suffix = 'TEST'
+        if not is_d3m_format(test_dataset, suffix):
+            convert_d3m_format(test_dataset, self.output_folder, self.problem_config, suffix)
+
+        dataset_in_container = '/input/dataset/TEST/dataset_TEST/datasetDoc.json'
         fitted_solution_id = list(model.keys())[0]
-        print('Testing model...')
-        tested_solution_path = self.ta3.do_test(fitted_solution_id, dataset_test_path)
-        print('Testing finished!')
-        tested_solution_path = tested_solution_path.replace('file:///output/', '')
-        predictions = pd.read_csv(join(self.output_folder, tested_solution_path))
+        logger.info('Testing model...')
+        predictions_path_in_container = self.ta3.do_test(fitted_solution_id, dataset_in_container)
+
+        if not predictions_path_in_container.startswith('file://'):
+            logger.error('Exposed output "%s" from TA2 cannot be read', predictions_path_in_container)
+            return
+        logger.info('Testing finished!')
+        predictions_path_in_container = predictions_path_in_container.replace('file:///output/', '')
+        predictions = pd.read_csv(join(self.output_folder, predictions_path_in_container))
 
         return predictions
 
-    def test_score(self, solution_id, dataset_path_test):
+    def test_score(self, solution_id, score_dataset):
         #  TODO: Use TA2TA3 API to score
-        pipeline_id = None
-        search_id = None
-        for pipeline in self.pipelines:
-            if solution_id == pipeline['id']:
-                pipeline_id = pipeline['json_representation']['id']
-                search_id = pipeline['search_id']
-                break
 
-        dataset_path = '/input/dataset/'
-        dataset_train_path = join(dataset_path, 'TRAIN/dataset_TRAIN/datasetDoc.json')
-        dataset_test_path = join(dataset_path, 'TEST/dataset_TEST/datasetDoc.json')
-        dataset_score_path = join(dataset_path, 'SCORE/dataset_SCORE/datasetDoc.json')
-        if not isfile(dataset_score_path):
-            dataset_score_path = join(dataset_path, 'SCORE/dataset_TEST/datasetDoc.json')
-        problem_path = join(dataset_path, 'TRAIN/problem_TRAIN/problemDoc.json')
+        if solution_id not in self.pipelines:
+            logger.error('Pipeline id=%s does not exist' % solution_id)
+            return
+
+        pipeline_id = self.pipelines[solution_id]['json_representation']['id']
+        search_id = self.pipelines[solution_id]['search_id']
+
+        dataset_in_container = '/input/dataset/'
+        dataset_train_path = join(dataset_in_container, 'TRAIN/dataset_TRAIN/datasetDoc.json')
+        dataset_test_path = join(dataset_in_container, 'TEST/dataset_TEST/datasetDoc.json')
+        dataset_score_path = join(dataset_in_container, 'SCORE/dataset_SCORE/datasetDoc.json')
+        problem_path = join(dataset_in_container, 'TRAIN/problem_TRAIN/problemDoc.json')
         pipeline_path = join('/output/', search_id, 'pipelines_searched', '%s.json' % pipeline_id)
         score_pipeline_path = join('/output/', 'fit_score_%s.csv' % pipeline_id)
         metric = None
@@ -138,8 +151,8 @@ class Automl:
             df = pd.read_csv(join(self.output_folder, 'fit_score_%s.csv' % pipeline_id))
             score = round(df['value'][0], 5)
             metric = df['metric'][0].lower()
-        except Exception as e:
-            print('Error calculating test score', e)
+        except:
+            logger.exception('Error calculating test score')
 
         return metric, score
 
@@ -147,7 +160,7 @@ class Automl:
         profiler_inputs = []
         pipeline_ids = set()
 
-        for pipeline in self.pipelines:
+        for pipeline in self.pipelines.values():
             if pipeline['id'] not in pipeline_ids:
                 pipeline_ids.add(pipeline['id'])
                 if 'digest' not in pipeline['json_representation']:
@@ -166,23 +179,22 @@ class Automl:
                     'start': pipeline['json_representation']['created'],
                     'end': pipeline['found_time'],
                     'scores': pipeline['score'],
-                    'pipeline_source': {'name': self.ta2},
+                    'pipeline_source': {'name': self.ta2_id},
                 }
                 profiler_inputs.append(profiler_data)
 
             else:
-                print('Ignoring repeated pipeline id=%s' % pipeline['id'])
+                logger.warning('Ignoring repeated pipeline id=%s' % pipeline['id'])
 
         return profiler_inputs
 
-    def start_ta2(self, dataset_path, output_path):
-        print('Initializing TA2...')
+    def start_ta2(self):
+        logger.info('Initializing TA2...')
 
         process = subprocess.Popen(['docker', 'stop', 'ta2_container'])
         process.wait()
 
-        dataset_path = split(dataset_path)[0]
-        self.container = subprocess.Popen(
+        self.ta2 = subprocess.Popen(
             [
                 'docker', 'run', '--rm',
                 '--name', 'ta2_container',
@@ -190,9 +202,10 @@ class Automl:
                 '-e', 'D3MRUN=ta2ta3',
                 '-e', 'D3MINPUTDIR=/input',
                 '-e', 'D3MOUTPUTDIR=/output',
-                '-v', '%s:/input/dataset/' % dataset_path,
-                '-v', '%s:/output' % output_path,
-                TA2_DOCKER_IMAGES[self.ta2]
+                '-e', 'D3MSTATICDIR=/output',  # TODO: Temporal assignment for D3MSTATICDIR env variable
+                '-v', '%s:/input/dataset/' % self.dataset,
+                '-v', '%s:/output' % self.output_folder,
+                TA2_DOCKER_IMAGES[self.ta2_id]
             ]
         )
         time.sleep(4)  # Wait for TA2
@@ -200,7 +213,7 @@ class Automl:
             try:
                 self.ta3 = BasicTA3()
                 self.ta3.do_hello()
-                print('TA2 initialized!')
+                logger.info('TA2 initialized!')
                 break
             except:
                 if self.ta3.channel is not None:
@@ -210,34 +223,28 @@ class Automl:
                 time.sleep(4)
 
     def end_session(self):
-        print('Ending session...')
-        if self.container is not None:
+        logger.info('Ending session...')
+        if self.ta2 is not None:
             process = subprocess.Popen(['docker', 'stop', 'ta2_container'])
             process.wait()
 
-        print('Session ended!')
+        logger.info('Session ended!')
 
     def get_summary_pipeline(self, pipeline_json):
-        primitives = []
+        primitives_summary = []
         for primitive in pipeline_json['steps']:
             primitive_name = primitive['primitive']['python_path']
             if primitive_name not in IGNORE_SUMMARY_PRIMITIVES:
-                primitive_name = '.'.join(primitive_name.split('.')[-2:]).lower()
-                primitives.append(primitive_name)
+                primitive_name_short = '.'.join(primitive_name.split('.')[-2:]).lower()
+                if primitive_name_short not in primitives_summary:
+                    primitives_summary.append(primitive_name_short)
 
-        return ', '.join(primitives)
+        return ', '.join(primitives_summary)
+
+    @staticmethod
+    def add_new_ta2(name, docker_image):
+        TA2_DOCKER_IMAGES[name] = docker_image
+        logger.info('TA2 "%s" added!', name)
 
 
-if __name__ == '__main__':
-    output_path = '/Users/rlopez/D3M/tmp/'
-    train_dataset = '/Users/rlopez/D3M/datasets/seed_datasets_current/185_baseball/TRAIN'
-    test_dataset = '/Users/rlopez/D3M/datasets/seed_datasets_current/185_baseball/TEST'
-
-    automl = Automl(output_path, 'CMU')
-    pipelines = automl.search_pipelines(train_dataset, time_bound=1)
-    model = automl.train(pipelines[0]['id'])
-    predictions = automl.test(model, test_dataset)
-    automl.create_profiler_inputs()
-    score = automl.test_score(pipelines[0]['id'], test_dataset)
-    automl.end_session()
 
